@@ -12,10 +12,11 @@ class OkeraException(Exception):
 class OkeraCredentialProvider(CredentialProvider):
     CANONICAL_NAME = "okera-aws-creds"
 
-    def __init__(self, rest_api_url, token):
+    def __init__(self, rest_api_url, token, verify_ssl=False):
         super().__init__()
         self._okera_rest_api_url = rest_api_url.rstrip('/')
         self._okera_token = token
+        self._verify_ssl = verify_ssl
 
     def load(self):
         creds = DeferredRefreshableCredentials(refresh_using=self._refresh, method="sts-assume-role")
@@ -34,18 +35,29 @@ class OkeraCredentialProvider(CredentialProvider):
     def _custom_aws_cred_refresh(self):
         api_url = "%s/%s" % (self._okera_rest_api_url, "api/v2/aws-tokens")
         headers = {"Authorization": "Bearer %s" % self._okera_token}
-        res = requests.post(api_url, headers=headers)
-        if res.status_code in (401, 403):
+        res = requests.post(api_url, headers=headers, verify=self._verify_ssl)
+        if res.status_code in (401, 403, 405):
             raise OkeraException("Error in authenticating credentials request: %s" % res.text)
         else:
             return res.json()
 
 def okera_session(session, token, rest_uri, proxy_uri):
-    bc_session = session._session
+    bc_session = None
+    if isinstance(session, botocore.session.Session):
+        bc_session = session
+    else:
+        bc_session = session._session
+
     boto3_session = session
 
-    cred_provider = bc_session.get_component('credential_provider')
-    cred_provider.insert_before('env', OkeraCredentialProvider(rest_uri, token))
+    # TODO: don't assume it's a cred resolver
+    cred_chain = bc_session.get_component('credential_provider')
+    okera_cred_provider = OkeraCredentialProvider(rest_uri, token)
+    if cred_chain.providers:
+        first_provider = cred_chain.providers[0].METHOD
+        cred_chain.insert_before(first_provider, okera_cred_provider)
+    else:
+        cred_chain.providers.insert(0, okera_cred_provider)
     config = botocore.config.Config(
         proxies={'https': proxy_uri})
 
