@@ -3,7 +3,6 @@
 # pylint: disable=len-as-condition
 # pylint: disable=protected-access
 # pylint: disable=too-few-public-methods
-# pylint: disable=too-many-arguments
 # pylint: disable=too-many-instance-attributes
 # pylint: disable=too-many-locals
 # pylint: disable=unidiomatic-typecheck
@@ -14,18 +13,37 @@ import json
 import math
 import os
 import random
+import statistics
 import string
+import time
 import unittest
 
+import pytz
+
 from okera import context, _thrift_api
 from okera._thrift_api import TTypeId, TConfigType
 
-DEFAULT_PRESTO_HOST = 'localhost'
-DEFAULT_PRESTO_PORT = 8049
+DEFAULT_PRESTO_HOST = os.environ['ODAS_TEST_HOST']
+DEFAULT_PRESTO_PORT = os.environ['ODAS_TEST_PORT_PRESTO_COORD_HTTPS']
+DEFAULT_REST_SERVER_HOST = os.environ['ODAS_TEST_HOST']
+DEFAULT_REST_SERVER_PORT = os.environ['ODAS_TEST_PORT_REST_SERVER']
+DEFAULT_ACCESS_PROXY_HOST = os.environ['ODAS_TEST_HOST']
+DEFAULT_ACCESS_PROXY_PORT = os.environ['ODAS_TEST_PORT_ACCESS_PROXY']
+
+TEST_LEVELS = {
+    'notests' : 0,
+    'smoke' : 1,
+    'dev' : 2,
+    'checkin' : 3,
+    'ci' : 4,
+    'all' : 5,
+}
 
 TEST_LEVEL = 'smoke'
 if 'TEST_LEVEL' in os.environ:
     TEST_LEVEL = os.environ['TEST_LEVEL']
+elif 'CEREBRO_TEST_LEVEL' in os.environ:
+    TEST_LEVEL = os.environ['CEREBRO_TEST_LEVEL']
 ROOT_TOKEN = os.environ['OKERA_HOME'] + '/integration/tokens/cerebro.token'
 
 identity = lambda x: x
@@ -38,11 +56,16 @@ def get_env_var(name, coercer, default):
 def get_bool_env_var(name, default):
     return get_env_var(name, lambda x: str(x).lower() in ['true'], default)
 
-def get_test_context(auth_mech=None):
+def test_level_lt(min_level):
+    if TEST_LEVEL not in TEST_LEVELS:
+        return False
+    return TEST_LEVELS[TEST_LEVEL] < TEST_LEVELS[min_level]
+
+def get_test_context(auth_mech=None, dialect=None, namespace='okera', tz=pytz.utc):
     if auth_mech is None:
         auth_mech = get_env_var('PYCEREBRO_TEST_AUTH_MECH', identity, 'NOSASL')
 
-    ctx = context()
+    ctx = context(dialect=dialect, namespace=namespace, tz=tz)
     if auth_mech == 'NOSASL':
         ctx.disable_auth()
     elif auth_mech == 'TOKEN':
@@ -51,43 +74,73 @@ def get_test_context(auth_mech=None):
         ctx.disable_auth()
     return ctx
 
-def get_planner(host=None, port=None, dialect='okera'):
+def get_planner(ctx, host=None, port=None, dialect='okera', presto_port=None,
+                namespace=None):
     if host is not None:
         host = host
     else:
-        host = get_env_var('PYCEREBRO_TEST_HOST', identity, 'localhost')
+        host = get_env_var('ODAS_TEST_HOST', identity, 'localhost')
 
     if port is not None:
         port = port
     else:
-        port = get_env_var('PYCEREBRO_TEST_PLANNER_PORT', int, 12050)
+        port = get_env_var('ODAS_TEST_PORT_PLANNER_THRIFT', int, 12050)
     if 'presto' in dialect:
-        ctx = get_test_context()
         ctx.enable_token_auth(token_str='root')
         return ctx.connect(host=host,
                            port=port,
                            presto_host=host,
-                           presto_port=DEFAULT_PRESTO_PORT)
-    return get_test_context().connect(host=host, port=port)
+                           presto_port=DEFAULT_PRESTO_PORT,
+                           namespace=namespace)
+    return ctx.connect(host=host, port=port, presto_port=presto_port, namespace=namespace)
 
-def get_worker(host=None, port=None):
+def get_worker(ctx, host=None, port=None):
     if host is not None:
         host = host
     else:
-        host = get_env_var('PYCEREBRO_TEST_HOST', identity, 'localhost')
+        host = get_env_var('ODAS_TEST_HOST', identity, 'localhost')
 
     if port is not None:
         port = port
     else:
-        port = get_env_var('PYCEREBRO_TEST_WORKER_PORT', int, 13050)
+        port = get_env_var('ODAS_TEST_PORT_WORKER_THRIFT', int, 13050)
 
-    return get_test_context().connect_worker(host=host, port=port)
+    return ctx.connect_worker(host=host, port=port)
+
+def get_rest_server_url(scheme='http', host=DEFAULT_REST_SERVER_HOST, port=DEFAULT_REST_SERVER_PORT):
+    return "%s://%s:%s" % (scheme, host, port)
+
+def get_access_proxy_url(scheme='http', host=DEFAULT_ACCESS_PROXY_HOST, port=DEFAULT_ACCESS_PROXY_PORT):
+    return "%s://%s:%s" % (scheme, host, port)
+
+def measure_latency(iters, fn, msg=None, quiet=False):
+    durations = []
+    for _ in range(0, iters):
+        start = time.time()
+        fn()
+        durations.append((time.time() - start) * 1000)
+    durations = sorted(durations)
+    if quiet:
+        return durations
+    if msg:
+      print("\n%s\n    Iterations: %s" % (msg, iters))
+    else:
+      print("\nIterations " + str(iters))
+    print("Mean " + str(statistics.mean(durations)) + " ms")
+    print("50%: " + str(durations[int(len(durations) * .5)]) + " ms")
+    print("90%: " + str(durations[int(len(durations) * .90)]) + " ms")
+    print("95%: " + str(durations[int(len(durations) * .95)]) + " ms")
+    print("99%: " + str(durations[int(len(durations) * .99)]) + " ms")
+    print("99.5%: " + str(durations[int(len(durations) * .995)]) + " ms")
+    print("99.9%: " + str(durations[int(len(durations) * .999)]) + " ms")
+    return durations
 
 def configure_botocore_patch():
     os.environ['OKERA_PATCH_BOTO'] = 'True'
-    os.environ['OKERA_PLANNER_HOST'] = 'localhost'
-    with open(ROOT_TOKEN, 'r') as token_file:
-        os.environ['OKERA_USER_TOKEN'] = token_file.read().strip(' \t\n\r')
+    os.environ['OKERA_PLANNER_HOST'] = \
+        get_env_var('ODAS_TEST_HOST', identity, 'localhost')
+    os.environ['OKERA_PLANNER_PORT'] = \
+        get_env_var('ODAS_TEST_PORT_PLANNER_THRIFT', identity, 12050)
     from okera import initialize_default_context, check_and_patch_botocore
     initialize_default_context()
     check_and_patch_botocore()
@@ -162,6 +215,41 @@ def list_configs(conn, config_type):
         raise ValueError("Invalid config type.")
     return conn.scan_as_json(table_name)
 
+class SchemaNode():
+    def __init__(self):
+        self.col = None
+        self.children = []
+
+    @staticmethod
+    def convert(cols, idx, schema):
+        node = SchemaNode()
+        node.col = cols[idx]
+        schema.append(node)
+        idx += 1
+        if node.col.type.num_children:
+            for _ in range(0, node.col.type.num_children):
+                idx = SchemaNode.convert(cols, idx, node.children)
+        return idx
+
+class TmpView():
+    def __init__(self, conn, sql):
+        self.conn = conn
+        # TODO: make this more unique so can be used concurrently
+        self.db = "test_tmp_db"
+        self.view = "tmp_view"
+        self.conn.execute_ddl("CREATE DATABASE IF NOT EXISTS %s" % self.db)
+        self.conn.execute_ddl("DROP VIEW IF EXISTS %s.%s" % (self.db, self.view))
+        self.conn.execute_ddl("CREATE VIEW %s.%s AS %s" % (self.db, self.view, sql))
+
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        self.conn.execute_ddl("DROP VIEW IF EXISTS %s.%s" % (self.db, self.view))
+
+    def name(self):
+        return '%s.%s' % (self.db, self.view)
+
 class JsonGeneratorNode():
     def __init__(self):
         self.types = []
@@ -374,6 +462,87 @@ class TestBase(unittest.TestCase):
     def _ddl_count(conn, sql):
         return len(conn.execute_ddl(sql))
 
+    @staticmethod
+    def _recreate_test_role(conn, role, users):
+        conn.execute_ddl("DROP ROLE IF EXISTS %s" % role)
+        conn.execute_ddl("CREATE ROLE %s" % role)
+        for user in users:
+            conn.execute_ddl("GRANT ROLE %s TO GROUP %s" % (role, user))
+
+    @staticmethod
+    def _recreate_test_db(conn, db):
+        conn.execute_ddl("DROP DATABASE IF EXISTS %s CASCADE" % db)
+        conn.execute_ddl("CREATE DATABASE %s" % db)
+
+    @staticmethod
+    def _create_all_types(conn, db, include_date=False):
+        ddl = '''
+            CREATE EXTERNAL TABLE %s.alltypes(
+              bool_col BOOLEAN,
+              tinyint_col TINYINT,
+              smallint_col SMALLINT,
+              int_col INT,
+              bigint_col BIGINT,
+              float_col FLOAT,
+              double_col DOUBLE,
+              string_col STRING,
+              varchar_col VARCHAR(10),
+              char_col CHAR(5),
+        '''
+        if include_date:
+            ddl += 'date_col DATE,\n'
+        ddl += '''
+              timestamp_col TIMESTAMP,
+              decimal_col decimal(24, 10)
+            )
+            ROW FORMAT DELIMITED FIELDS TERMINATED BY '|'
+            STORED AS TEXTFILE
+            LOCATION 's3://cerebrodata-test/alltypes/'
+        '''
+        conn.execute_ddl(ddl % db)
+
+    @staticmethod
+    def _tag_all_columns(conn, db, dataset, ns, key, create_tag=False):
+        if create_tag:
+            conn.delete_attribute(ns, key)
+            conn.create_attribute(ns, key)
+        ds = conn.list_datasets(db, name=dataset)[0]
+        for col in ds.schema.cols:
+            if col.name:
+                conn.assign_attribute(ns, key, db, dataset, col.name)
+
+    @staticmethod
+    def get_random_leaf_column(conn, db, tbl):
+        """ Returns a random path to a leaf col in this table."""
+        tbl = conn.list_datasets(db, name=tbl)[0]
+        schema = TestBase.convert_schema(tbl.schema.cols)
+        result = ''
+        flattened = ''
+        while schema:
+            col = random.choice(schema)
+            if col.col.name:
+                if result:
+                    result += '.'
+                if flattened:
+                    flattened += '__'
+                result += col.col.name.lower()
+                flattened += col.col.name.lower()
+            # else:
+            #     if flattened:
+            #         flattened += '__'
+            #     flattened += 'item'
+            schema = col.children
+        return result, flattened
+
+    @staticmethod
+    def convert_schema(cols):
+        """ Converts a flattened inorder schema list to a tree """
+        schema = []
+        idx = 0
+        while idx < len(cols):
+            idx = SchemaNode.convert(cols, idx, schema)
+        return schema
+
     @staticmethod
     def _visible_cols(cols):
         result = []
@@ -391,6 +560,27 @@ class TestBase(unittest.TestCase):
                 total_children += c.type.num_children
         return len(cols) - total_children
 
+    @staticmethod
+    def collect_column_attributes(ds):
+        result = {}
+        for col in ds.schema.cols:
+            if not col.attribute_values:
+                continue
+            for v in col.attribute_values:
+                key = v.database
+                if v.table:
+                    key += '.' + v.table
+                    if v.column:
+                        key += '.' + v.column
+                if key not in result:
+                    result[key] = []
+                result[key].append(
+                    v.attribute.attribute_namespace + '.' + v.attribute.key)
+        for _, v in result.items():
+            v.sort()
+        return result
+
+
     @staticmethod
     def _try_parse_datetime(v):
         if not isinstance(v, str):
@@ -423,14 +613,15 @@ class TestBase(unittest.TestCase):
         except:
             return False
 
-    def _equals(self, v1, v2):
-        if self._is_float(v1) and self._is_float(v2):
+    @staticmethod
+    def _equals(v1, v2):
+        if TestBase._is_float(v1) and TestBase._is_float(v2):
             v1 = float(v1)
             v2 = float(v2)
 
         if isinstance(v1, float) and isinstance(v2, float):
             return v1 == v2 or (math.isnan(v1) and math.isnan(v2)) \
-                or abs(v1 - v2) < .00001
+                or abs(v1 - v2) < .0001
         if v1 != v2:
             # Try as datetime with different formats
             d1 = TestBase._try_parse_datetime(v1)
@@ -441,25 +632,27 @@ class TestBase(unittest.TestCase):
             print("Types: %s != %s" % (type(v1), type(v2)))
         return v1 == v2
 
-    def _is_empty_dictionary(self, v):
+    @staticmethod
+    def _is_empty_dictionary(v):
         """ Checks if v is None or a dictionary (recursively) of None values """
         if v is None:
             return True
         if type(v) != dict:
             return False
         for _, val in v.items():
-            if not self._is_empty_dictionary(val):
+            if not TestBase._is_empty_dictionary(val):
                 return False
         return True
 
-    def __deep_compare(self, actual, expected, allow_missing, empty_struct_equals_null,
-                       required_type):
+    @staticmethod
+    def __deep_compare(actual, expected, allow_missing, empty_struct_equals_null,
+                       required_type, empty_array_equals_null, zero_equals_none):
         if empty_struct_equals_null:
             if type(actual) == dict and (expected is None or len(expected) == 0):
                 # We don't support nullable structs, so instead every field in the
                 # struct is None
                 for k, v in actual.items():
-                    if not self._is_empty_dictionary(v):
+                    if not TestBase._is_empty_dictionary(v):
                         print("actual has a non-null struct. Expecting null.")
                         return False
                 return True
@@ -468,6 +661,11 @@ class TestBase(unittest.TestCase):
                 # fields is otherwise not possible.
                 return True
 
+        # For parquet file format, the actual is empty for an empty list.
+        if empty_array_equals_null:
+            if type(expected) == list and (actual is None):
+                return True
+
         # As we are strongly typed, we will convert the types to the "bigger" type for
         # schema merge cases.
         if type(actual) == str and type(expected) in [int, float]:
@@ -492,11 +690,18 @@ class TestBase(unittest.TestCase):
             if type(actual) in [bool, float, int] and type(expected) == dict \
                     and not expected:
                 return True
+            if type(actual) == float and math.isnan(actual) and expected is None:
+                return True
+            if actual == "" and expected is None and zero_equals_none:
+                return True
+            if type(actual) == type(None) and type(expected) in [float, int]:
+                if zero_equals_none and expected in [0, 0.0]:
+                    return True
             if required_type and type(expected) in [bool, float, int, str]:
                 if type(expected) != required_type:
                     # The required type doesn't match so the expected should be None
                     if actual is not None and \
-                            not self._equals(actual, required_type(expected)):
+                            not TestBase._equals(actual, required_type(expected)):
                         print("Expecting actual to be None. %s, %s" % (actual, expected))
                         return False
                     return True
@@ -510,17 +715,19 @@ class TestBase(unittest.TestCase):
                 # but is null might not exist in the raw file version, but only if
                 # allow_missing is set
                 if k in expected:
-                    if not self.__deep_compare(v, expected[k], allow_missing,
-                                               empty_struct_equals_null, required_type):
+                    if not TestBase.__deep_compare(
+                            v, expected[k], allow_missing,
+                            empty_struct_equals_null, required_type,
+                            empty_array_equals_null, zero_equals_none):
                         print("Key %s from expected is not in actual." % k)
                         print("%s != %s" % (v, expected[k]))
                         return False
                 elif k not in expected and not \
-                        (self._is_empty_dictionary(v) and allow_missing):
+                        (TestBase._is_empty_dictionary(v) and allow_missing):
                     print("Key %s from actual is not in expected." % k)
                     return False
             for k, v in expected.items():
-                if k not in actual:
+                if k not in actual and not allow_missing:
                     print("Key %s from expected is not in actual." % k)
                     return False
         elif type(actual) == list:
@@ -529,45 +736,54 @@ class TestBase(unittest.TestCase):
                       (len(actual), len(expected)))
                 return False
             for idx in range(len(actual)):
-                if not self.__deep_compare(actual[idx], expected[idx], allow_missing,
-                                           empty_struct_equals_null, required_type):
+                if not TestBase.__deep_compare(
+                        actual[idx], expected[idx], allow_missing,
+                        empty_struct_equals_null, required_type,
+                        empty_array_equals_null, zero_equals_none):
                     print("List elements don't match at idx %s. %s != %s" % \
                           (idx, actual[idx], expected[idx]))
                     return False
         else:
-            return self._equals(actual, expected)
+            return TestBase._equals(actual, expected)
 
         return True
 
-    def _lower_keys(self, x):
+    @staticmethod
+    def _lower_keys(x):
         if isinstance(x, list):
-            return [self._lower_keys(v) for v in x]
+            return [TestBase._lower_keys(v) for v in x]
         if isinstance(x, dict):
-            return dict((k.lower(), self._lower_keys(v)) for k, v in x.items())
+            return dict((k.lower(), TestBase._lower_keys(v)) for k, v in x.items())
         return x
 
-    def compare_json(self, actual, expected, allow_missing, empty_struct_equals_null,
-                     batch_mode, required_type):
-        actual = self._lower_keys(actual)
-        expected = self._lower_keys(expected)
+    @staticmethod
+    def compare_json(asserter, actual, expected, allow_missing, empty_struct_equals_null,
+                     batch_mode, required_type, empty_array_equals_null,
+                     zero_equals_none=False):
+        actual = TestBase._lower_keys(actual)
+        expected = TestBase._lower_keys(expected)
 
         if batch_mode and len(actual) != len(expected):
             print("Failure: Lengths did not match %s != %s" % \
                 (len(actual), len(expected)))
             return False
-        self.assertEqual(len(actual), len(expected))
+        asserter.assertEqual(len(actual), len(expected))
 
         for idx in range(len(actual)):
             obj1 = actual[idx]
             obj2 = expected[idx]
 
-            if self.__deep_compare(obj1, obj2, allow_missing, empty_struct_equals_null,
-                                   required_type):
+            if TestBase.__deep_compare(
+                    obj1, obj2, allow_missing, empty_struct_equals_null,
+                    required_type, empty_array_equals_null, zero_equals_none):
                 continue
             if batch_mode:
                 return False
             print("EXPECTED:\n%s" % json.dumps(expected, indent=2, sort_keys=True))
             print("\nACTUAL:\n%s" % json.dumps(actual, indent=2, sort_keys=True))
-            self.assertEqual(json.dumps(actual, indent=2, sort_keys=True),
-                             json.dumps(expected, indent=2, sort_keys=True))
+            asserter.assertEqual(json.dumps(actual, indent=2, sort_keys=True),
+                                 json.dumps(expected, indent=2, sort_keys=True))
         return True
+
+def random_string(length):
+    return ''.join(random.choice(string.ascii_uppercase) for _ in range(length))
