@@ -66,8 +66,8 @@ class ListDatasetsTest(unittest.TestCase):
         with common.get_planner(ctx) as conn:
             conn.execute_ddl('DROP DATABASE IF EXISTS %s CASCADE' % DB1)
             conn.execute_ddl('DROP DATABASE IF EXISTS %s CASCADE' % DB2)
-            conn.execute_ddl('DROP DATACONNECTION %s' % PSQL_CXN)
-            conn.execute_ddl('DROP DATACONNECTION %s' % MYSQL_CXN)
+            conn.execute_ddl('DROP DATACONNECTION IF EXISTS %s' % PSQL_CXN)
+            conn.execute_ddl('DROP DATACONNECTION IF EXISTS %s' % MYSQL_CXN)
             conn.execute_ddl(CREATE_PSQL_CXN)
             conn.execute_ddl(CREATE_MYSQL_CXN)
 
