@@ -265,24 +265,7 @@ class BasicTest(common.TestBase):
         ctx = common.get_test_context()
         with common.get_planner(ctx) as conn:
             self._recreate_test_db(conn, DB)
-            conn.execute_ddl('''
-                CREATE EXTERNAL TABLE %s.alltypes(
-                  bool_col BOOLEAN,
-                  tinyint_col TINYINT,
-                  smallint_col SMALLINT,
-                  int_col INT,
-                  bigint_col BIGINT,
-                  float_col FLOAT,
-                  double_col DOUBLE,
-                  string_col STRING,
-                  varchar_col VARCHAR(10),
-                  char_col CHAR(5),
-                  timestamp_col TIMESTAMP,
-                  decimal_col decimal(24, 10))
-                ROW FORMAT DELIMITED FIELDS TERMINATED BY '|'
-                STORED AS TEXTFILE
-                LOCATION 's3://cerebrodata-test/alltypes/'
-                ''' % DB)
+            self._create_all_types(conn, DB)
 
             # Verify schema from catalog, plan and data are all timestamp
             schema = conn.plan('%s.alltypes' % DB).schema
@@ -383,6 +366,31 @@ class BasicTest(common.TestBase):
             self.assertEqual(2, len(res))
             self.assertEqual('"123', res[1]['c2'])
 
+    def test_scan_delta_dummy_metadata(self):
+        DB = "delta_dummy_scan_db"
+        ctx = common.get_test_context()
+        with common.get_planner(ctx) as conn:
+            self._recreate_test_db(conn, DB)
+            conn.execute_ddl('''
+      CREATE EXTERNAL TABLE %s.airline_events_delta (
+          col array<string>)
+      WITH SERDEPROPERTIES (
+          'path'='s3://cerebrodata-test/delta/airlines/airline_events/',
+          'serialization.format'='1')
+      STORED AS TEXTFILE
+      LOCATION 's3://cerebrodata-test/delta/airlines/airline_events/'
+      TBLPROPERTIES (
+          'totalSize'='66153', 'numRows'='-1', 'rawDataSize'='-1',
+          'okera.delta.infer-schema'='true',
+          'COLUMN_STATS_ACCURATE'='false',
+          'spark.sql.sources.schema.part.0'='{\"type\":\"struct\",\"fields\":[]}',
+          'numFiles'='1', 'spark.sql.partitionProvider'='catalog',
+          'spark.sql.sources.schema.numParts'='1', 'spark.sql.sources.provider'='delta',
+          'spark.sql.create.version'='3.0.1')
+                ''' % DB)
+            res = conn.scan_as_json('%s.airline_events_delta' % DB)
+            self.assertEqual(1326, len(res))
+
     def test_pandas_index(self):
         ctx = common.get_test_context()
         with common.get_planner(ctx) as planner:
